module ActionSegue
  module SegueRegistration
    extend ActiveSupport::Concern

    module ClassMethods
      def register_segue(action:, as:)
        before_action only: action do
          session[:action_segues] ||= ActiveSupport::OrderedHash.new
          Rails.logger.debug "[ActionSegue] registering new segue as '#{as}' with params: #{params}"
          session[:action_segues][as] = Segue.new(id: as, params: params)
        end
      end
    end

    # Completes the segue and returns either the params to redirect to the segue start point,
    # all the fallback if the segue is not in progress.
    def complete_segue_url(id, fallback:)
      if session[:action_segues].present? && session[:action_segues][id].present?
        segue = clear_segue id
        Rails.logger.debug "[ActionSegue] completed '#{id}'"
        url_for segue.params
      else
        Rails.logger.debug "[ActionSegue] no segue matched '#{id}'"
        fallback
      end
    end

    # Remove a segue from the stack
    def clear_segue(id)
      Rails.logger.debug "[ActionSegue] clearing segue '#{id}'"
      session[:action_segues].delete id
    end

    # Completes the last segue, regardless of what it was
    def complete_current_segue_url(fallback:)
      complete_segue_url session[:action_segues].try(:keys).try(:last), fallback: fallback
    end

  end
end
